@@ -25,8 +25,8 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "app/logging.h"
 #include "archive_kerfuffle.h"
+#include "ark_debug.h"
 #include "archiveinterface.h"
 #include "jobs.h"
 
@@ -40,16 +40,16 @@
 
 #include <KPluginLoader>
 #include <KMimeTypeTrader>
-#include <KServiceTypeTrader>
 
-Q_LOGGING_CATEGORY(KERFUFFLE, "ark.kerfuffle", QtWarningMsg)
-
-static bool comparePlugins(const KService::Ptr &p1, const KService::Ptr &p2)
+namespace Kerfuffle
 {
-    return (p1->property(QLatin1String( "X-KDE-Priority" )).toInt()) > (p2->property(QLatin1String( "X-KDE-Priority" )).toInt());
+
+bool Archive::comparePlugins(const KService::Ptr &p1, const KService::Ptr &p2)
+{
+    return (p1->property(QStringLiteral( "X-KDE-Priority" )).toInt()) > (p2->property(QStringLiteral( "X-KDE-Priority" )).toInt());
 }
 
-static QString determineMimeType(const QString& filename)
+QString Archive::determineMimeType(const QString& filename)
 {
     QMimeDatabase db;
 
@@ -61,7 +61,7 @@ static QString determineMimeType(const QString& filename)
     if (fileinfo.completeSuffix().toLower().remove(QRegularExpression(QStringLiteral("[^a-z\\.]"))).contains(QStringLiteral("tar."))) {
         inputFile.chop(fileinfo.completeSuffix().length());
         inputFile += fileinfo.completeSuffix().remove(QRegularExpression(QStringLiteral("[^a-zA-Z\\.]")));
-        qCDebug(KERFUFFLE) << "Validated filename of compressed tar" << filename << "into filename" << inputFile;
+        qCDebug(ARK) << "Validated filename of compressed tar" << filename << "into filename" << inputFile;
     }
 
     QMimeType mimeFromExtension = db.mimeTypeForFile(inputFile, QMimeDatabase::MatchExtension);
@@ -76,53 +76,51 @@ static QString determineMimeType(const QString& filename)
     // Compressed tar-archives are detected as single compressed files when
     // detecting by content. The following code prevents tar.gz, tar.bz2 and
     // tar.xz files being opened using the singlefile plugin.
-    if ((mimeFromExtension == db.mimeTypeForName("application/x-compressed-tar") &&
-         mimeFromContent == db.mimeTypeForName("application/gzip")) ||
-        (mimeFromExtension == db.mimeTypeForName("application/x-bzip-compressed-tar") &&
-         mimeFromContent == db.mimeTypeForName("application/x-bzip")) ||
-        (mimeFromExtension == db.mimeTypeForName("application/x-xz-compressed-tar") &&
-         mimeFromContent == db.mimeTypeForName("application/x-xz"))) {
+    if ((mimeFromExtension == db.mimeTypeForName(QStringLiteral("application/x-compressed-tar")) &&
+         mimeFromContent == db.mimeTypeForName(QStringLiteral("application/gzip"))) ||
+        (mimeFromExtension == db.mimeTypeForName(QStringLiteral("application/x-bzip-compressed-tar")) &&
+         mimeFromContent == db.mimeTypeForName(QStringLiteral("application/x-bzip"))) ||
+        (mimeFromExtension == db.mimeTypeForName(QStringLiteral("application/x-xz-compressed-tar")) &&
+         mimeFromContent == db.mimeTypeForName(QStringLiteral("application/x-xz")))) {
         return mimeFromExtension.name();
     }
 
     if (mimeFromExtension != mimeFromContent) {
 
-        // #354344: ISO files are currently not detected-by-content.
-        if (mimeFromExtension.inherits(QStringLiteral("application/x-cd-image"))) {
+        if (mimeFromContent == db.mimeTypeForName(QStringLiteral("application/octet-stream"))) {
+            qCWarning(ARK) << "Could not detect mimetype from content."
+                           << "Using extension-based mimetype:" << mimeFromExtension.name();
             return mimeFromExtension.name();
         }
 
-        qCWarning(KERFUFFLE) << "Mimetype for filename extension (" << mimeFromExtension.name()
-                             << ") did not match mimetype for content (" << mimeFromContent.name()
-                             << "). Using content-based mimetype.";
+        qCWarning(ARK) << "Mimetype for filename extension (" << mimeFromExtension.name()
+                       << ") did not match mimetype for content (" << mimeFromContent.name()
+                       << "). Using content-based mimetype.";
     }
 
     return mimeFromContent.name();
 }
 
-static KService::List findPluginOffers(const QString& filename, const QString& fixedMimeType)
+KService::List Archive::findPluginOffers(const QString& filename, const QString& fixedMimeType)
 {
     KService::List offers;
 
-    qCDebug(KERFUFFLE) << "Find plugin offers for" << filename << "with mime" << fixedMimeType;
+    qCDebug(ARK) << "Find plugin offers for" << filename << "with mime" << fixedMimeType;
 
     const QString mimeType = fixedMimeType.isEmpty() ? determineMimeType(filename) : fixedMimeType;
 
-    qCDebug(KERFUFFLE) << "Detected mime" << mimeType;
+    qCDebug(ARK) << "Detected mime" << mimeType;
 
     if (!mimeType.isEmpty()) {
-        offers = KMimeTypeTrader::self()->query(mimeType, QLatin1String( "Kerfuffle/Plugin" ), QLatin1String( "(exist Library)" ));
+        offers = KMimeTypeTrader::self()->query(mimeType, QStringLiteral( "Kerfuffle/Plugin" ), QStringLiteral( "(exist Library)" ));
         qSort(offers.begin(), offers.end(), comparePlugins);
     }
 
-    qCDebug(KERFUFFLE) << "Have" << offers.count() << "offers";
+    qCDebug(ARK) << "Have" << offers.count() << "offers";
 
     return offers;
 }
 
-namespace Kerfuffle
-{
-
 QDebug operator<<(QDebug d, const fileRootNodePair &pair)
 {
     d.nospace() << "fileRootNodePair(" << pair.file << "," << pair.rootNode << ")";
@@ -136,46 +134,75 @@ Archive *Archive::create(const QString &fileName, QObject *parent)
 
 Archive *Archive::create(const QString &fileName, const QString &fixedMimeType, QObject *parent)
 {
-    qCDebug(KERFUFFLE) << "Going to create archive" << fileName;
+    qCDebug(ARK) << "Going to create archive" << fileName;
 
     qRegisterMetaType<ArchiveEntry>("ArchiveEntry");
 
-    const KService::List offers = findPluginOffers(fileName, fixedMimeType);
-
-    if (offers.isEmpty()) {
-        qCWarning(KERFUFFLE) << "Could not find a plugin to handle" << fileName;
-        return Q_NULLPTR;
-    }
-
-    const QString pluginName = offers.first()->library();
-    qCDebug(KERFUFFLE) << "Loading plugin" << pluginName;
-
-    KPluginFactory * const factory = KPluginLoader(pluginName).factory();
-    if (!factory) {
-        qCWarning(KERFUFFLE) << "Invalid plugin factory for" << pluginName;
-        return Q_NULLPTR;
-    }
-
     QVariantList args;
     args.append(QVariant(QFileInfo(fileName).absoluteFilePath()));
 
-    ReadOnlyArchiveInterface * const iface = factory->create<ReadOnlyArchiveInterface>(0, args);
-    if (!iface) {
-        qCWarning(KERFUFFLE) << "Could not create plugin instance" << pluginName << "for" << fileName;
-        return Q_NULLPTR;
+    const KService::List offers = findPluginOffers(fileName, fixedMimeType);
+    if (offers.isEmpty()) {
+        qCCritical(ARK) << "Could not find a plugin to handle" << fileName;
+        return new Archive(NoPlugin, parent);
     }
 
-    return new Archive(iface, parent);
+    KPluginFactory *factory;
+    ReadOnlyArchiveInterface *iface;
+
+    foreach (KService::Ptr service, offers) {
+
+        QString pluginName = service->library();
+        bool isReadOnly = !service->property(QStringLiteral("X-KDE-Kerfuffle-ReadWrite")).toBool();
+        qCDebug(ARK) << "Loading plugin" << pluginName;
+
+        factory = KPluginLoader(pluginName).factory();
+        if (!factory) {
+            qCWarning(ARK) << "Invalid plugin factory for" << pluginName;
+            continue;
+        }
+
+        iface = factory->create<ReadOnlyArchiveInterface>(0, args);
+        if (!iface) {
+            qCWarning(ARK) << "Could not create plugin instance" << pluginName;
+            continue;
+        }
+
+        if (iface->isCliBased()) {
+            qCDebug(ARK) << "Finding executables for plugin" << pluginName;
+
+            if (iface->findExecutables(!isReadOnly)) {
+                return new Archive(iface, isReadOnly, parent);
+            } else {
+                qCWarning(ARK) << "Failed to find needed executables for plugin" << pluginName;
+            }
+        } else {
+            // Not CliBased plugin, don't search for executables.
+            return new Archive(iface, isReadOnly, parent);
+        }
+    }
+
+    qCCritical(ARK) << "Failed to find a usable plugin for" << fileName;
+    return new Archive(FailedPlugin, parent);
 }
 
-Archive::Archive(ReadOnlyArchiveInterface *archiveInterface, QObject *parent)
-        : QObject(parent),
-        m_iface(archiveInterface),
-        m_hasBeenListed(false),
-        m_isPasswordProtected(false),
-        m_isSingleFolderArchive(false)
+Archive::Archive(ArchiveError errorCode, QObject *parent)
+        : QObject(parent)
+        , m_error(errorCode)
 {
-    qCDebug(KERFUFFLE) << "Created archive instance";
+    qCDebug(ARK) << "Created archive instance with error";
+}
+
+Archive::Archive(ReadOnlyArchiveInterface *archiveInterface, bool isReadOnly, QObject *parent)
+        : QObject(parent)
+        , m_iface(archiveInterface)
+        , m_hasBeenListed(false)
+        , m_isReadOnly(isReadOnly)
+        , m_isPasswordProtected(false)
+        , m_isSingleFolderArchive(false)
+        , m_error(NoError)
+{
+    qCDebug(ARK) << "Created archive instance";
 
     Q_ASSERT(archiveInterface);
     archiveInterface->setParent(this);
@@ -188,9 +215,19 @@ Archive::~Archive()
 {
 }
 
+bool Archive::isValid() const
+{
+    return (m_error == NoError);
+}
+
+ArchiveError Archive::error() const
+{
+    return m_error;
+}
+
 bool Archive::isReadOnly() const
 {
-    return m_iface->isReadOnly();
+    return (m_iface->isReadOnly() || m_isReadOnly);
 }
 
 KJob* Archive::open()
@@ -205,7 +242,7 @@ KJob* Archive::create()
 
 ListJob* Archive::list()
 {
-    qCDebug(KERFUFFLE) << "Going to list files";
+    qCDebug(ARK) << "Going to list files";
 
     ListJob *job = new ListJob(m_iface, this);
     job->setAutoDelete(false);
@@ -220,7 +257,7 @@ ListJob* Archive::list()
 
 DeleteJob* Archive::deleteFiles(const QList<QVariant> & files)
 {
-    qCDebug(KERFUFFLE) << "Going to delete files" << files;
+    qCDebug(ARK) << "Going to delete files" << files;
 
     if (m_iface->isReadOnly()) {
         return 0;
@@ -232,18 +269,18 @@ DeleteJob* Archive::deleteFiles(const QList<QVariant> & files)
 
 AddJob* Archive::addFiles(const QStringList & files, const CompressionOptions& options)
 {
-    qCDebug(KERFUFFLE) << "Going to add files" << files << "with options" << options;
+    qCDebug(ARK) << "Going to add files" << files << "with options" << options;
     Q_ASSERT(!m_iface->isReadOnly());
     AddJob *newJob = new AddJob(files, options, static_cast<ReadWriteArchiveInterface*>(m_iface), this);
     connect(newJob, &AddJob::result, this, &Archive::onAddFinished);
     return newJob;
 }
 
-ExtractJob* Archive::copyFiles(const QList<QVariant> & files, const QString & destinationDir, ExtractionOptions options)
+ExtractJob* Archive::copyFiles(const QList<QVariant>& files, const QString& destinationDir, const ExtractionOptions& options)
 {
     ExtractionOptions newOptions = options;
     if (isPasswordProtected()) {
-        newOptions[QLatin1String( "PasswordProtectedHint" )] = true;
+        newOptions[QStringLiteral( "PasswordProtectedHint" )] = true;
     }
 
     ExtractJob *newJob = new ExtractJob(files, destinationDir, newOptions, m_iface, this);
@@ -322,6 +359,11 @@ bool Archive::isPasswordProtected()
     return m_isPasswordProtected;
 }
 
+QString Archive::comment() const
+{
+    return m_iface->comment();
+}
+
 QString Archive::subfolderName()
 {
     listIfNotListed();
@@ -338,102 +380,78 @@ void Archive::enableHeaderEncryption(bool enable)
     m_iface->setHeaderEncryptionEnabled(enable);
 }
 
-QStringList supportedMimeTypes()
+QSet<QString> supportedMimeTypes()
 {
-    const QLatin1String constraint("(exist Library)");
     const QLatin1String basePartService("Kerfuffle/Plugin");
+    const KService::List offers = KServiceTypeTrader::self()->query(basePartService,
+                                                                    QStringLiteral("(exist Library)"));
+    QSet<QString> supported;
 
-    const KService::List offers = KServiceTypeTrader::self()->query(basePartService, constraint);
-    KService::List::ConstIterator it = offers.constBegin();
-    KService::List::ConstIterator itEnd = offers.constEnd();
-
-    QStringList supported;
-
-    for (; it != itEnd; ++it) {
-        KService::Ptr service = *it;
-        QStringList mimeTypes = service->serviceTypes();
-
-        foreach (const QString& mimeType, mimeTypes) {
-            if (mimeType != basePartService && !supported.contains(mimeType)) {
-                supported.append(mimeType);
+    foreach (const KService::Ptr& service, offers) {
+        foreach (const QString& mimeType, service->serviceTypes()) {
+            if (mimeType != basePartService) {
+                supported.insert(mimeType);
             }
         }
     }
 
-    qCDebug(KERFUFFLE) << "Returning supported mimetypes" << supported;
+    qCDebug(ARK) << "Returning supported mimetypes" << supported;
 
     return supported;
 }
 
-QStringList supportedWriteMimeTypes()
+QSet<QString> supportedWriteMimeTypes()
 {
-    const QLatin1String constraint("(exist Library) and ([X-KDE-Kerfuffle-ReadWrite] == true)");
     const QLatin1String basePartService("Kerfuffle/Plugin");
+    const KService::List offers = KServiceTypeTrader::self()->query(basePartService,
+                                                                    QStringLiteral("(exist Library) and ([X-KDE-Kerfuffle-ReadWrite] == true)"));
+    QSet<QString> supported;
 
-    const KService::List offers = KServiceTypeTrader::self()->query(basePartService, constraint);
-    KService::List::ConstIterator it = offers.constBegin();
-    KService::List::ConstIterator itEnd = offers.constEnd();
-
-    QStringList supported;
-
-    for (; it != itEnd; ++it) {
-        KService::Ptr service = *it;
-        QStringList mimeTypes = service->serviceTypes();
-
-        foreach (const QString& mimeType, mimeTypes) {
-            if (mimeType != basePartService && !supported.contains(mimeType)) {
-                supported.append(mimeType);
+    foreach (const KService::Ptr& service, offers) {
+        foreach (const QString& mimeType, service->serviceTypes()) {
+            if (mimeType != basePartService) {
+                supported.insert(mimeType);
             }
         }
     }
 
-    qCDebug(KERFUFFLE) << "Returning supported write mimetypes" << supported;
+    qCDebug(ARK) << "Returning supported write mimetypes" << supported;
 
     return supported;
 }
 
 QSet<QString> supportedEncryptEntriesMimeTypes()
 {
-    const QLatin1String constraint("(exist Library) and ([X-KDE-Kerfuffle-EncryptEntries] == true)");
-    const QLatin1String basePartService("Kerfuffle/Plugin");
-
-    const KService::List offers = KServiceTypeTrader::self()->query(basePartService, constraint);
+    const KService::List offers = KServiceTypeTrader::self()->query(QStringLiteral("Kerfuffle/Plugin"),
+                                                                    QStringLiteral("(exist Library)"));
     QSet<QString> supported;
 
     foreach (const KService::Ptr& service, offers) {
-        QStringList mimeTypes = service->serviceTypes();
-
-        foreach (const QString& mimeType, mimeTypes) {
-            if (mimeType != basePartService) {
-                supported.insert(mimeType);
-            }
+        QStringList list(service->property(QStringLiteral("X-KDE-Kerfuffle-EncryptEntries")).toStringList());
+        foreach (const QString& mimeType, list) {
+            supported.insert(mimeType);
         }
     }
 
-    qCDebug(KERFUFFLE) << "Returning supported mimetypes" << supported;
+    qCDebug(ARK) << "Entry encryption supported for mimetypes" << supported;
 
     return supported;
 }
 
 QSet<QString> supportedEncryptHeaderMimeTypes()
 {
-    const QLatin1String constraint("(exist Library) and ([X-KDE-Kerfuffle-EncryptHeader] == true)");
-    const QLatin1String basePartService("Kerfuffle/Plugin");
-
-    const KService::List offers = KServiceTypeTrader::self()->query(basePartService, constraint);
+    const KService::List offers = KServiceTypeTrader::self()->query(QStringLiteral("Kerfuffle/Plugin"),
+                                                                    QStringLiteral("(exist Library)"));
     QSet<QString> supported;
 
     foreach (const KService::Ptr& service, offers) {
-        QStringList mimeTypes = service->serviceTypes();
-
-        foreach (const QString& mimeType, mimeTypes) {
-            if (mimeType != basePartService) {
-                supported.insert(mimeType);
-            }
+        QStringList list(service->property(QStringLiteral("X-KDE-Kerfuffle-EncryptHeader")).toStringList());
+        foreach (const QString& mimeType, list) {
+            supported.insert(mimeType);
         }
     }
 
-    qCDebug(KERFUFFLE) << "Returning supported mimetypes" << supported;
+    qCDebug(ARK) << "Header encryption supported for mimetypes" << supported;
 
     return supported;
 }
