@@ -31,7 +31,7 @@
 #include "archiveinterface.h"
 #include "kerfuffle_export.h"
 #include <QtCore/QProcess>
-#include <QtCore/QRegExp>
+#include <QRegularExpression>
 
 class KProcess;
 class KPtyProcess;
@@ -74,6 +74,11 @@ enum CliInterfaceParameters {
      * $Archive - the path of the archive
      */
     ListArgs,
+    /**
+     * QStringList (default empty)
+     * List of regexp patterns that indicate a corrupt archive.
+     */
+    CorruptArchivePatterns,
 
     ///////////////[ EXTRACT ]/////////////
 
@@ -141,9 +146,9 @@ enum CliInterfaceParameters {
      */
     PasswordSwitch,
     /**
-     * QString
-     * This is a regexp, defining how to recognize the last line in a "File
-     * already exists" prompt when extracting.
+     * QStringList
+     * This is a stringlist with regexps, defining how to recognize the last
+     * line in a "File already exists" prompt when extracting.
      */
     FileExistsExpression,
     /**
@@ -263,16 +268,18 @@ public:
     virtual ~CliInterface();
 
     virtual bool list() Q_DECL_OVERRIDE;
-    virtual bool copyFiles(const QList<QVariant> & files, const QString & destinationDirectory, ExtractionOptions options) Q_DECL_OVERRIDE;
+    virtual bool copyFiles(const QList<QVariant>& files, const QString& destinationDirectory, const ExtractionOptions& options) Q_DECL_OVERRIDE;
     virtual bool addFiles(const QStringList & files, const CompressionOptions& options) Q_DECL_OVERRIDE;
     virtual bool deleteFiles(const QList<QVariant> & files) Q_DECL_OVERRIDE;
 
+    virtual void resetParsing() = 0;
     virtual ParameterList parameterList() const = 0;
     virtual bool readListLine(const QString &line) = 0;
-
     bool doKill() Q_DECL_OVERRIDE;
     bool doSuspend() Q_DECL_OVERRIDE;
     bool doResume() Q_DECL_OVERRIDE;
+    bool findExecutables(bool isReadWrite) Q_DECL_OVERRIDE;
+    bool isCliBased() const Q_DECL_OVERRIDE;
 
     /**
      * Returns the list of characters which are preceded by a
@@ -317,7 +324,6 @@ private:
 
     bool checkForPasswordPromptMessage(const QString& line);
 
-    bool checkForFileExistsMessage(const QString& line);
     bool handleFileExistsMessage(const QString& filename);
     bool checkForErrorMessage(const QString& line, int parameterIndex);
     void handleLine(const QString& line);
@@ -355,9 +361,8 @@ private:
     bool moveToFinalDest(const QVariantList &files, const QString &finalDest, const QTemporaryDir &tmpDir);
 
     QByteArray m_stdOutData;
-    QRegExp m_existsPattern;
-    QRegExp m_passwordPromptPattern;
-    QHash<int, QList<QRegExp> > m_patternCache;
+    QRegularExpression m_passwordPromptPattern;
+    QHash<int, QList<QRegularExpression> > m_patternCache;
 
 #ifdef Q_OS_WIN
     KProcess *m_process;
