@@ -21,8 +21,8 @@
  *
  */
 
-#include "app/logging.h"
 #include "archivemodel.h"
+#include "ark_debug.h"
 #include "kerfuffle/archive_kerfuffle.h"
 #include "kerfuffle/jobs.h"
 
@@ -80,9 +80,11 @@ public:
 
         QMimeDatabase db;
         if (entry[IsDirectory].toBool()) {
-            m_icon = QIcon::fromTheme(db.mimeTypeForName("inode/directory").iconName()).pixmap(IconSize(KIconLoader::Small), IconSize(KIconLoader::Small));
+            m_icon = QIcon::fromTheme(db.mimeTypeForName(QStringLiteral("inode/directory")).iconName()).pixmap(IconSize(KIconLoader::Small),
+                                                                                                               IconSize(KIconLoader::Small));
         } else {
-            m_icon = QIcon::fromTheme(db.mimeTypeForFile(m_entry[FileName].toString()).iconName()).pixmap(IconSize(KIconLoader::Small), IconSize(KIconLoader::Small));
+            m_icon = QIcon::fromTheme(db.mimeTypeForFile(m_entry[FileName].toString()).iconName()).pixmap(IconSize(KIconLoader::Small),
+                                                                                                          IconSize(KIconLoader::Small));
         }
     }
 
@@ -336,7 +338,7 @@ QVariant ArchiveModel::data(const QModelIndex &index, int role) const
                         return QVariant();
                     } else {
                         int ratio = int(100 * ((double)size - compressedSize) / size);
-                        return QString(QString::number(ratio) + QLatin1String( " %" ));
+                        return QString(QString::number(ratio) + QStringLiteral(" %"));
                     }
                 }
 
@@ -382,7 +384,7 @@ QVariant ArchiveModel::headerData(int section, Qt::Orientation, int role) const
 {
     if (role == Qt::DisplayRole) {
         if (section >= m_showColumns.size()) {
-            qCDebug(PART) << "WEIRD: showColumns.size = " << m_showColumns.size()
+            qCDebug(ARK) << "WEIRD: showColumns.size = " << m_showColumns.size()
             << " and section = " << section;
             return QVariant();
         }
@@ -556,13 +558,13 @@ QStringList ArchiveModel::mimeTypes() const
     QStringList types;
 
     // MIME types we accept for dragging (eg. Dolphin -> Ark).
-    types << QLatin1String("text/uri-list")
-          << QLatin1String("text/plain")
-          << QLatin1String("text/x-moz-url");
+    types << QStringLiteral("text/uri-list")
+          << QStringLiteral("text/plain")
+          << QStringLiteral("text/x-moz-url");
 
     // MIME types we accept for dropping (eg. Ark -> Dolphin).
-    types << QLatin1String("application/x-kde-ark-dndextract-service")
-          << QLatin1String("application/x-kde-ark-dndextract-path");
+    types << QStringLiteral("application/x-kde-ark-dndextract-service")
+          << QStringLiteral("application/x-kde-ark-dndextract-path");
 
     return types;
 }
@@ -572,9 +574,9 @@ QMimeData *ArchiveModel::mimeData(const QModelIndexList &indexes) const
     Q_UNUSED(indexes)
 
     QMimeData *mimeData = new QMimeData;
-    mimeData->setData(QLatin1String("application/x-kde-ark-dndextract-service"),
+    mimeData->setData(QStringLiteral("application/x-kde-ark-dndextract-service"),
                       QDBusConnection::sessionBus().baseService().toUtf8());
-    mimeData->setData(QLatin1String("application/x-kde-ark-dndextract-path"),
+    mimeData->setData(QStringLiteral("application/x-kde-ark-dndextract-path"),
                       m_dbusPathName.toUtf8());
 
     return mimeData;
@@ -603,14 +605,14 @@ bool ArchiveModel::dropMimeData(const QMimeData * data, Qt::DropAction action, i
     if (parent.isValid()) {
         QModelIndex droppedOnto = index(row, column, parent);
         if (entryForIndex(droppedOnto).value(IsDirectory).toBool()) {
-            qCDebug(PART) << "Using entry";
+            qCDebug(ARK) << "Using entry";
             path = entryForIndex(droppedOnto).value(FileName).toString();
         } else {
             path = entryForIndex(parent).value(FileName).toString();
         }
     }
 
-    qCDebug(PART) << "Dropped onto " << path;
+    qCDebug(ARK) << "Dropped onto " << path;
 
 #endif
 
@@ -619,11 +621,15 @@ bool ArchiveModel::dropMimeData(const QMimeData * data, Qt::DropAction action, i
     return true;
 }
 
-// For a rationale, see bugs #194241 and #241967
+// For a rationale, see bugs #194241, #241967 and #355839
 QString ArchiveModel::cleanFileName(const QString& fileName)
 {
-    if ((fileName == QLatin1String("/")) ||
-        (fileName == QLatin1String("."))) { // "." is present in ISO files
+    // Skip entries with filename "/" or "//" or "."
+    // "." is present in ISO files
+    QRegularExpression pattern(QStringLiteral("/+|\\."));
+    QRegularExpressionMatch match;
+    if (fileName.contains(pattern, &match) && match.captured() == fileName) {
+        qCDebug(ARK) << "Skipping entry with filename" << fileName;
         return QString();
     } else if (fileName.startsWith(QLatin1String("./"))) {
         return fileName.mid(2);
@@ -669,6 +675,7 @@ ArchiveDirNode* ArchiveModel::parentFor(const ArchiveEntry& entry)
             ArchiveEntry e;
             e[ FileName ] = (parent == m_rootNode) ?
                             piece : parent->entry()[ FileName ].toString() + QLatin1Char( '/' ) + piece;
+            e[ InternalID ] = e.value(FileName);
             e[ IsDirectory ] = true;
             node = new ArchiveDirNode(parent, e);
             insertNode(node);
@@ -701,7 +708,7 @@ QModelIndex ArchiveModel::indexForNode(ArchiveNode *node)
 
 void ArchiveModel::slotEntryRemoved(const QString & path)
 {
-    qCDebug(PART) << "Removed node at path " << path;
+    qCDebug(ARK) << "Removed node at path " << path;
 
     const QString entryFileName(cleanFileName(path));
     if (entryFileName.isEmpty()) {
@@ -722,7 +729,7 @@ void ArchiveModel::slotEntryRemoved(const QString & path)
 
         endRemoveRows();
     } else {
-        qCDebug(PART) << "Did not find the removed node";
+        qCDebug(ARK) << "Did not find the removed node";
     }
 }
 
@@ -748,7 +755,7 @@ void ArchiveModel::slotNewEntry(const ArchiveEntry& entry)
 void ArchiveModel::newEntry(const ArchiveEntry& receivedEntry, InsertBehaviour behaviour)
 {
     if (receivedEntry[FileName].toString().isEmpty()) {
-        qCDebug(PART) << "Weird, received empty entry (no filename) - skipping";
+        qCDebug(ARK) << "Weird, received empty entry (no filename) - skipping";
         return;
     }
 
@@ -782,7 +789,7 @@ void ArchiveModel::newEntry(const ArchiveEntry& receivedEntry, InsertBehaviour b
         m_showColumns << toInsert;
         endInsertColumns();
 
-        qCDebug(PART) << "Showing columns: " << m_showColumns;
+        qCDebug(ARK) << "Showing columns: " << m_showColumns;
     }
 
     //make a copy
@@ -790,6 +797,7 @@ void ArchiveModel::newEntry(const ArchiveEntry& receivedEntry, InsertBehaviour b
 
     //#194241: Filenames such as "./file" should be displayed as "file"
     //#241967: Entries called "/" should be ignored
+    //#355839: Entries called "//" should be ignored
     QString entryFileName = cleanFileName(entry[FileName].toString());
     if (entryFileName.isEmpty()) { // The entry contains only "." or "./"
         return;
@@ -800,7 +808,7 @@ void ArchiveModel::newEntry(const ArchiveEntry& receivedEntry, InsertBehaviour b
     if (m_rootNode) {
         ArchiveNode *existing = m_rootNode->findByPath(entry[ FileName ].toString().split(QLatin1Char( '/' )));
         if (existing) {
-            qCDebug(PART) << "Refreshing entry for" << entry[FileName].toString();
+            qCDebug(ARK) << "Refreshing entry for" << entry[FileName].toString();
 
             // Multi-volume files are repeated at least in RAR archives.
             // In that case, we need to sum the compressed size for each volume
@@ -817,7 +825,8 @@ void ArchiveModel::newEntry(const ArchiveEntry& receivedEntry, InsertBehaviour b
     ArchiveDirNode *parent = parentFor(entry);
 
     /// 3. Create an ArchiveNode
-    QString name = entry[ FileName ].toString().split(QLatin1Char( '/' ), QString::SkipEmptyParts).last();
+    const QStringList path = entry[FileName].toString().split(QLatin1Char('/'), QString::SkipEmptyParts);
+    const QString name = path.last();
     ArchiveNode *node = parent->find(name);
     if (node) {
         node->setEntry(entry);
@@ -842,7 +851,7 @@ void ArchiveModel::slotLoadingFinished(KJob *job)
     endResetModel();
     m_newArchiveEntries.clear();
 
-    qCDebug(PART) << "Added" << i << "entries to model";
+    qCDebug(ARK) << "Added" << i << "entries to model";
 
     emit loadingFinished(job);
 }
@@ -896,14 +905,14 @@ KJob* ArchiveModel::setArchive(Kerfuffle::Archive *archive)
     return job;
 }
 
-ExtractJob* ArchiveModel::extractFile(const QVariant& fileName, const QString & destinationDir, const Kerfuffle::ExtractionOptions options) const
+ExtractJob* ArchiveModel::extractFile(const QVariant& fileName, const QString& destinationDir, const Kerfuffle::ExtractionOptions& options) const
 {
     QList<QVariant> files;
     files << QVariant::fromValue(fileRootNodePair(fileName.toString()));
     return extractFiles(files, destinationDir, options);
 }
 
-ExtractJob* ArchiveModel::extractFiles(const QList<QVariant>& files, const QString & destinationDir, const Kerfuffle::ExtractionOptions options) const
+ExtractJob* ArchiveModel::extractFiles(const QList<QVariant>& files, const QString& destinationDir, const Kerfuffle::ExtractionOptions& options) const
 {
     Q_ASSERT(m_archive);
     ExtractJob *newJob = m_archive->copyFiles(files, destinationDir, options);
@@ -969,7 +978,7 @@ void ArchiveModel::slotCleanupEmptyDirs()
     while (!queue.isEmpty()) {
         QPersistentModelIndex node = queue.takeFirst();
         ArchiveEntry entry = entryForIndex(node);
-        qCDebug(PART) << "Trying " << entry[FileName].toString();
+        qCDebug(ARK) << "Trying " << entry[FileName].toString();
 
         if (!hasChildren(node)) {
             if (!entry.contains(InternalID)) {
@@ -984,7 +993,7 @@ void ArchiveModel::slotCleanupEmptyDirs()
 
     foreach(const QPersistentModelIndex& node, nodesToDelete) {
         ArchiveNode *rawNode = static_cast<ArchiveNode*>(node.internalPointer());
-        qCDebug(PART) << "Delete with parent entries " << rawNode->parent()->entries() << " and row " << rawNode->row();
+        qCDebug(ARK) << "Delete with parent entries " << rawNode->parent()->entries() << " and row " << rawNode->row();
         beginRemoveRows(parent(node), rawNode->row(), rawNode->row());
         rawNode->parent()->removeEntryAt(rawNode->row());
         endRemoveRows();
