@@ -19,15 +19,14 @@
  *
  */
 
-#include "app/logging.h"
 #include "arkviewer.h"
+#include "ark_debug.h"
 
 #include <KLocalizedString>
 #include <KMimeTypeTrader>
 #include <KIconLoader>
 #include <KMessageBox>
 #include <KRun>
-#include <KHtml/KHTMLPart>
 #include <KSharedConfig>
 #include <KWindowConfig>
 
@@ -46,7 +45,7 @@
 ArkViewer::ArkViewer(QWidget *parent, Qt::WindowFlags flags)
         : QDialog(parent, flags)
 {
-    qCDebug(PART) << "ArkViewer opened";
+    qCDebug(ARK) << "ArkViewer opened";
 
     // Set a QVBoxLayout as main layout of dialog
     m_mainLayout = new QVBoxLayout(this);
@@ -57,7 +56,7 @@ ArkViewer::ArkViewer(QWidget *parent, Qt::WindowFlags flags)
     m_mainLayout->addWidget(buttonBox);
     buttonBox->button(QDialogButtonBox::Close)->setDefault(true);
     buttonBox->button(QDialogButtonBox::Close)->setShortcut(Qt::CTRL | Qt::Key_Return);
-    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
+    connect(buttonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
 
     connect(this, &ArkViewer::finished, this, &ArkViewer::dialogClosed);
 }
@@ -100,17 +99,17 @@ void ArkViewer::view(const QString& fileName, QWidget *parent)
 {
     QMimeDatabase db;
     QMimeType mimeType = db.mimeTypeForFile(fileName);
-    qCDebug(PART) << "viewing" << fileName << "with mime type:" << mimeType.name();
+    qCDebug(ARK) << "viewing" << fileName << "with mime type:" << mimeType.name();
     KService::Ptr viewer = ArkViewer::getViewer(mimeType.name());
 
     const bool needsExternalViewer = (viewer &&
-                                      !viewer->hasServiceType(QLatin1String("KParts/ReadOnlyPart")));
+                                      !viewer->hasServiceType(QStringLiteral("KParts/ReadOnlyPart")));
     if (needsExternalViewer) {
         // We have already resolved the MIME type and the service above.
         // So there is no point in using KRun::runUrl() which would need
         // to do the same again.
 
-        qCDebug(PART) << "Using external viewer";
+        qCDebug(ARK) << "Using external viewer";
 
         const QList<QUrl> fileUrlList = {QUrl::fromLocalFile(fileName)};
         // The last argument (tempFiles) set to true means that the temporary
@@ -119,13 +118,13 @@ void ArkViewer::view(const QString& fileName, QWidget *parent)
         return;
     }
 
-    qCDebug(PART) << "Using internal viewer";
+    qCDebug(ARK) << "Using internal viewer";
     bool viewInInternalViewer = true;
     if (!viewer) {
         // No internal viewer available for the file.  Ask the user if it
         // should be previewed as text/plain.
 
-        qCDebug(PART) << "Internal viewer not available";
+        qCDebug(ARK) << "Internal viewer not available";
 
         int response;
         if (!mimeType.isDefault()) {
@@ -135,7 +134,7 @@ void ArkViewer::view(const QString& fileName, QWidget *parent)
             response = KMessageBox::warningContinueCancel(parent,
                 xi18n("The internal viewer cannot preview this type of file<nl/>(%1).<nl/><nl/>Do you want to try to view it as plain text?", mimeType.name()),
                 i18nc("@title:window", "Cannot Preview File"),
-                KGuiItem(i18nc("@action:button", "Preview as Text"), QIcon::fromTheme(QLatin1String("text-plain"))),
+                KGuiItem(i18nc("@action:button", "Preview as Text"), QIcon::fromTheme(QStringLiteral("text-plain"))),
                 KStandardGuiItem::cancel(),
                 QStringLiteral("PreviewAsText_%1").arg(mimeType.name()));
         }
@@ -162,7 +161,7 @@ void ArkViewer::view(const QString& fileName, QWidget *parent)
     }
 
     if (viewInInternalViewer) {
-        qCDebug(PART) << "Opening internal viewer";
+        qCDebug(ARK) << "Opening internal viewer";
         ArkViewer *internalViewer = new ArkViewer(parent, Qt::Window);
         internalViewer->setModal(Qt::WindowModal);
         internalViewer->show();
@@ -181,7 +180,7 @@ void ArkViewer::view(const QString& fileName, QWidget *parent)
     // Only get here if there is no internal viewer available or could be
     // used for the file, and no external viewer was opened.  Nothing can be
     // done with the temporary file, so remove it now.
-    qCDebug(PART) << "Removing temporary file:" << fileName;
+    qCDebug(ARK) << "Removing temporary file:" << fileName;
     QFile::remove(fileName);
 }
 
@@ -226,51 +225,31 @@ bool ArkViewer::viewInInternalViewer(const QString& fileName, const QMimeType &m
 
     m_mainLayout->insertWidget(0, header);
 
-    // Insert the KPart into the main layout
-    m_part = KMimeTypeTrader::self()->createPartInstanceFromQuery<KParts::ReadOnlyPart>(mimeType.name(),
-                                                                                        this,
-                                                                                        this);
+    // Create the ReadOnlyPart instance.
+    m_part = KMimeTypeTrader::self()->createPartInstanceFromQuery<KParts::ReadOnlyPart>(mimeType.name(), this, this);
+
+    // Drop the KHTMLPart, if necessary.
+    const KService::Ptr service = KMimeTypeTrader::self()->preferredService(mimeType.name(), QStringLiteral("KParts/ReadOnlyPart"));
+    if (service.constData()->desktopEntryName() == QLatin1String("khtml")) {
+        KService::List offers = KMimeTypeTrader::self()->query(mimeType.name(), QStringLiteral("KParts/ReadOnlyPart"));
+        offers.removeFirst();
+        qCDebug(ARK) << "Removed KHTMLPart from the offers for mimetype" << mimeType.name()
+                     << ". Using" << offers.first().constData()->desktopEntryName() << "instead.";
+        m_part = offers.first().constData()->createInstance<KParts::ReadOnlyPart>(this, this);
+    }
 
     if (!m_part.data()) {
         return false;
     }
 
+    // Insert the KPart into the main layout.
     m_mainLayout->insertWidget(1, m_part.data()->widget());
 
-    if (m_part.data()->browserExtension()) {
-        connect(m_part.data()->browserExtension(), SIGNAL(openUrlRequestDelayed(QUrl,KParts::OpenUrlArguments,KParts::BrowserArguments)),
-                SLOT(slotOpenUrlRequestDelayed(QUrl,KParts::OpenUrlArguments,KParts::BrowserArguments)));
-    }
-
-    // #235546
-    // TODO: the user should be warned in a non-intrusive way that some features are going to be disabled
-    //       maybe there should be an option controlling this
-    KHTMLPart *khtmlPart = qobject_cast<KHTMLPart*>(m_part.data());
-    if (khtmlPart) {
-        qCDebug(PART) << "Disabling javascripts, plugins, java and external references for KHTMLPart";
-        khtmlPart->setJScriptEnabled(false);
-        khtmlPart->setJavaEnabled(false);
-        khtmlPart->setPluginsEnabled(false);
-        khtmlPart->setMetaRefreshEnabled(false);
-        khtmlPart->setOnlyLocalReferences(true);
-    }
-
     m_part.data()->openUrl(QUrl::fromLocalFile(fileName));
 
     return true;
 }
 
-void ArkViewer::slotOpenUrlRequestDelayed(const QUrl& url, const KParts::OpenUrlArguments& arguments, const KParts::BrowserArguments& browserArguments)
-{
-    qCDebug(PART) << "Opening URL: " << url;
-
-    Q_UNUSED(arguments)
-    Q_UNUSED(browserArguments)
-
-    KRun *runner = new KRun(url, 0, false);
-    runner->setRunExecutables(false);
-}
-
 KService::Ptr ArkViewer::getViewer(const QString &mimeType)
 {
     // No point in even trying to find anything for application/octet-stream
